///////////////////////////////////////////////////////////////////////
// ADSResourceMonitor.cpp
//
// Applied Data Systems
// ------------------------------------------------------------------
// 20-1139x
//
// Description
// -----------
// ADSResourceMonitor is a simple dialog-based application that provides graphical
// feedback on the current device resources.  It displays and allows adjustment of
// the current RAM allocation.  It also display percent utilization of the storage
// storage and memory RAM as well as the FlashFX Disk.
//
// ADSResourceMonitor also provides an onscreen "heartbeat" to allow a user to 
// easily identify system lockups.
//
//
// History
// -------
// 20-11391 - Dec 19, 2002 : ctacke
//   - Initial implementation

#include <windows.h>
#include "resource.h"
#include "commctrl.h"

extern "C" BOOL GetSystemMemoryDivision(LPDWORD lpdwStoragePages, 
										LPDWORD lpdwRAMPages, 
										LPDWORD lpdwPageSize);

extern "C" DWORD SetSystemMemoryDivision(DWORD dwStoragePages);

BOOL CALLBACK MainWndProc(HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lParam);
void CALLBACK MainTimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime );
void UpdateInfo();

HWND hWndMain;
HWND hWndMemLoad;
HWND hWndFileLoad;
HWND hWndFlashLoad;
HWND hWndDivision;
HWND hWndHeartbeat;

DWORD	storage;
DWORD	ram;
DWORD	pagesize;
STORE_INFORMATION si;

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	MSG		msg;
	UINT	hTimer;

	InitCommonControls();

	// create the dialog
	hWndMain = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_MONITOR), NULL, MainWndProc);

	// store handles that we'll be updating
	hWndMemLoad = GetDlgItem(hWndMain, IDC_MEM_LOAD);
	hWndDivision = GetDlgItem(hWndMain, IDC_DIVISION);
	hWndFileLoad = GetDlgItem(hWndMain, IDC_FILE_LOAD);
	hWndFlashLoad = GetDlgItem(hWndMain, IDC_FLASH_LOAD);
	hWndHeartbeat = GetDlgItem(hWndMain, IDC_HEARTBEAT);

	// initialize our global variables
	GetSystemMemoryDivision(&storage, &ram, &pagesize);
	GetStoreInformation(&si);

	// set the ranges for the sliders and progress bars
	SendMessage(hWndMemLoad, PBM_SETRANGE, NULL, MAKELPARAM(0, 100));
	SendMessage(hWndFileLoad, PBM_SETRANGE, NULL, MAKELPARAM(0, 100));
	SendMessage(hWndDivision, TBM_SETRANGE, TRUE, MAKELPARAM(0, storage + ram));
	SendMessage(hWndDivision, TBM_SETTICFREQ, (storage + ram) / 200, 0);
	SendMessage(hWndDivision, TBM_SETLINESIZE, (storage + ram) / 100, 0);
	SendMessage(hWndDivision, TBM_SETPAGESIZE, (storage + ram) / 100, 0);

	SendMessage(hWndDivision, TBM_SETPOS, TRUE, storage);

	// update the UI
	UpdateInfo();

	// show the dialog
	ShowWindow(hWndMain, SW_SHOW);
	UpdateWindow(hWndMain);

	// start the update timer
	hTimer = SetTimer(NULL, NULL, 250, MainTimerProc);

	// message pump
	while(GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return 0;
}

BOOL CALLBACK MainWndProc(HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
	static NMHDR nm;
	static DWORD div;	

	switch(wMsg)
	{
		case WM_CLOSE:
			// exit the app
			PostMessage(NULL, WM_QUIT, 0, 0);
		break;
		case WM_NOTIFY:
			// handle slider mesages
			memcpy(&nm, (void *)lParam, sizeof(nm));
			if(nm.idFrom == IDC_DIVISION)
			{
				storage = SendMessage(hWndDivision, TBM_GETPOS, 0, 0);
				DEBUGMSG(TRUE, (_T("new div=%i\r\n"), storage));
			}
		break;
	}
	
	return FALSE;
}

void CALLBACK MainTimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime )
{
	// update the global variables
	SetSystemMemoryDivision((DWORD)storage);
	GetSystemMemoryDivision(&storage, &ram, &pagesize);
	GetStoreInformation(&si);

	// and update the UI
	SendMessage(hWndDivision, TBM_SETPOS, TRUE, storage);

	UpdateInfo();

	// redraw the main window
	UpdateWindow(hWndMain);
}

void UpdateInfo()
{
	static MEMORYSTATUS ms;
	static ULARGE_INTEGER bytesavail;
	static ULARGE_INTEGER bytestotal;
	static ULARGE_INTEGER bytesfree;
	static UINT heartbeat = 0;
	static TCHAR label[16];

	GlobalMemoryStatus(&ms);
	GetDiskFreeSpaceEx(_T("\\FlashFX Disk"), &bytesavail, &bytestotal, &bytesfree);

	SetDlgItemInt(hWndMain, IDC_STORAGE, storage * pagesize / 1024, FALSE);
	SetDlgItemInt(hWndMain, IDC_RAM, ram * pagesize / 1024, FALSE);

	SetDlgItemInt(hWndMain, IDC_PHYS_AVAIL, ms.dwAvailPhys / 1024, FALSE);
	SetDlgItemInt(hWndMain, IDC_FILE_AVAIL, si.dwFreeSize / 1024, FALSE);
	SetDlgItemInt(hWndMain, IDC_FILE_SIZE, si.dwStoreSize / 1024, FALSE);
	
	_stprintf(label, _T("%i%%"), ms.dwMemoryLoad);
	SetDlgItemText(hWndMain, IDC_MEM_PCT, label);


	SendMessage(hWndMemLoad, PBM_SETPOS, ms.dwMemoryLoad, NULL);

	SendMessage(hWndFileLoad, PBM_SETPOS, (si.dwStoreSize - si.dwFreeSize) * 100 / si.dwStoreSize, NULL);
	_stprintf(label, _T("%i%%"), (si.dwStoreSize - si.dwFreeSize) * 100 / si.dwStoreSize);
	SetDlgItemText(hWndMain, IDC_FILE_PCT, label);

	_stprintf(label, _T("%i%%"), (bytestotal.LowPart - bytesavail.LowPart) * 100 / bytestotal.LowPart);
	SetDlgItemText(hWndMain, IDC_FLASH_PCT, label);

	SendMessage(hWndFlashLoad, PBM_SETPOS, (bytestotal.LowPart - bytesavail.LowPart) * 100 / bytestotal.LowPart, NULL);

	SendMessage(hWndHeartbeat, PBM_SETPOS, ms.dwMemoryLoad, NULL);

	SendMessage(hWndHeartbeat, BM_SETCHECK, heartbeat++ % 6 < 3 ? 1 : 0, 0);
}
